#include "CHidIO.h"
#include <tchar.h>
#include <vector>
//#include "dbt.h"

extern "C" {
#include "setupapi.h"
#include "hidsdi.h"
}

#pragma comment(lib, "hid.lib")
#pragma comment(lib, "Setupapi.lib")

HANDLE CHidIO::GetActiveHandle() const
{
    return m_hWriteHandle[m_szActiveDeviceIndex];
}

CHidIO::CHidIO()
    : m_bUseTwoHandle(TRUE)
    , m_hAbordEvent(CreateEvent(NULL,TRUE,FALSE,NULL))
    , m_hReadEvent(CreateEvent(NULL,TRUE,FALSE,NULL))
    , m_hWriteEvent(CreateEvent(NULL,TRUE,FALSE,NULL))
    , m_szActiveDeviceIndex(0)
    , m_hReadHandle()
    , m_hWriteHandle()
{
}
CHidIO::~CHidIO()
{
    CloseDevice();
    CloseHandle(m_hWriteEvent);
    CloseHandle(m_hReadEvent);
    CloseHandle(m_hAbordEvent);
}

size_t CHidIO::GetDeviceLength() const
{
    return m_hReadHandle.size();
}

size_t CHidIO::SetActiveDevice(size_t szIndex)
{
    m_DeviceData.Close();

    size_t szOldIndex = m_szActiveDeviceIndex;
    m_szActiveDeviceIndex = szIndex;

    if(m_szActiveDeviceIndex < m_hReadHandle.size()) {
        std::basic_string<TCHAR> path = m_sNames[m_szActiveDeviceIndex].c_str();
        for(size_t i = 0; i < path.size(); ++i) {
            if(!::isalnum(path[i]))
                path[i] = _T('_');
        }
        m_DeviceData.SetName(path.c_str());
        m_DeviceData.Open();
    }
    return szOldIndex;
}

size_t CHidIO::GetActiveDevice() const
{
    return m_szActiveDeviceIndex;
}

ShareArea<CHidShare> &CHidIO::GetActiveDeviceData()
{
    return m_DeviceData;
}

void CHidIO::CloseDevice()
{
    if(m_bUseTwoHandle) {
        size_t i;

        for(i = 0; i < m_hReadHandle.size(); ++i) {
            if(m_hReadHandle[i] != INVALID_HANDLE_VALUE)
                CancelIo(m_hReadHandle[i]);
        }

        for(i = 0; i < m_hWriteHandle.size(); ++i) {
            if(m_hWriteHandle[i] != INVALID_HANDLE_VALUE)
                CancelIo(m_hWriteHandle[i]);
        }

        for(i = 0; i < m_hReadHandle.size(); ++i) {
            if(m_hReadHandle[i] != INVALID_HANDLE_VALUE) {
                CloseHandle(m_hReadHandle[i]);
                m_hReadHandle[i] = INVALID_HANDLE_VALUE;
            }
        }

        for(i = 0; i < m_hWriteHandle.size(); ++i) {
            if(m_hWriteHandle[i] != INVALID_HANDLE_VALUE) {
                CloseHandle(m_hWriteHandle[i]);
                m_hWriteHandle[i] = INVALID_HANDLE_VALUE;
            }
        }
    } else {
        size_t i;
        for(i = 0; i < m_hReadHandle.size(); ++i) {
            if(m_hReadHandle[i] != INVALID_HANDLE_VALUE)
                CancelIo(m_hReadHandle[i]);
        }

        for(i = 0; i < m_hReadHandle.size(); ++i) {
            if(m_hReadHandle[i] != INVALID_HANDLE_VALUE) {
                CloseHandle(m_hReadHandle[i]);
                m_hReadHandle[i] = INVALID_HANDLE_VALUE;
            }
        }
    }
    m_hReadHandle.clear();
    m_hWriteHandle.clear();
    m_sNames.clear();
}

void CHidIO::CloseInactiveDevice()
{
    if(m_bUseTwoHandle) {
        size_t i;

        for(i = 0; i < m_hReadHandle.size(); ++i) {
            if(m_hReadHandle[i] != INVALID_HANDLE_VALUE
                    && m_szActiveDeviceIndex != i)
                CancelIo(m_hReadHandle[i]);
        }

        for(i = 0; i < m_hWriteHandle.size(); ++i) {
            if(m_hWriteHandle[i] != INVALID_HANDLE_VALUE
                    && m_szActiveDeviceIndex != i)
                CancelIo(m_hWriteHandle[i]);
        }

        for(i = 0; i < m_hReadHandle.size(); ++i) {
            if(m_hReadHandle[i] != INVALID_HANDLE_VALUE
                    && m_szActiveDeviceIndex != i) {
                CloseHandle(m_hReadHandle[i]);
                m_hReadHandle[i] = INVALID_HANDLE_VALUE;
            }
        }

        for(i = 0; i < m_hWriteHandle.size(); ++i) {
            if(m_hWriteHandle[i] != INVALID_HANDLE_VALUE
                    && m_szActiveDeviceIndex != i) {
                CloseHandle(m_hWriteHandle[i]);
                m_hWriteHandle[i] = INVALID_HANDLE_VALUE;
            }
        }
    } else {
        size_t i;
        for(i = 0; i < m_hReadHandle.size(); ++i
                && m_szActiveDeviceIndex != i) {
            if(m_hReadHandle[i] != INVALID_HANDLE_VALUE)
                CancelIo(m_hReadHandle[i]);
        }

        for(i = 0; i < m_hReadHandle.size(); ++i
                && m_szActiveDeviceIndex != i) {
            if(m_hReadHandle[i] != INVALID_HANDLE_VALUE) {
                CloseHandle(m_hReadHandle[i]);
                m_hReadHandle[i] = INVALID_HANDLE_VALUE;
            }
        }
    }
}

BOOL CHidIO::OpenDevice(BOOL bUseTwoHandle, USHORT usVID, USHORT usPID0, USHORT usPID1, USHORT usPID2)
{
    m_bUseTwoHandle = bUseTwoHandle;

    //CString MyDevPathName="";
    TCHAR MyDevPathName[MAX_PATH];

    //wq@GUIDcHidGuidӫOsHID]ƪfGUIDC
    GUID HidGuid;
    //wq@DEVINFOy`hDevInfoSetӫOs쪺]ƫHXy`C
    HDEVINFO hDevInfoSet;
    //wqMemberIndexAܷejĴXӳ]ơA0ܲĤ@ӳ]ơC
    DWORD MemberIndex;
    //DevInterfaceDataAΨӫOs]ƪXʱfH
    SP_DEVICE_INTERFACE_DATA DevInterfaceData;
    //wq@BOOLܶqAOsƽեάO_^\
    BOOL Result;
    //wq@RequiredSizeܶqAΨӱݭnOsԲӫHwĪסC
    DWORD RequiredSize;
    //wq@ӫV]ƸԲӫHcwC
    PSP_DEVICE_INTERFACE_DETAIL_DATA	pDevDetailData;
    //wq@ӥΨӫOs}]ƪy`C
    HANDLE hDevHandle;
    HANDLE hReadHandle;
    HANDLE hWriteHandle;
    //wq@HIDD_ATTRIBUTEScܶqAOs]ƪݩʡC
    HIDD_ATTRIBUTES DevAttributes;

    //lƳ]ƥ
    BOOL MyDevFound=FALSE;

    //lŪBgy`Lĥy`C
    m_hReadHandle.clear();
    m_hWriteHandle.clear();
    m_sNames.clear();

    //DevInterfaceDatac骺cbSizelƬcjp
    DevInterfaceData.cbSize=sizeof(DevInterfaceData);
    //DevAttributesc骺SizelƬcjp
    DevAttributes.Size=sizeof(DevAttributes);

    //եHidD_GetHidGuidHID]ƪGUIDAëOsbHidGuidC
    HidD_GetHidGuid(&HidGuid);

    //ھHidGuid]ƫHXC䤤FlagsѼƳ]m
    //DIGCF_DEVICEINTERFACE|DIGCF_PRESENTAe̪ܨϥΪGUID
    //fGUIDA̪ܥuC|bϥΪ]ơA]ڭ̳o̥u
    //dwgsW]ơC^y`OsbhDevinfoC`N]
    //HXbϥΧAnϥΨSetupDiDestroyDeviceInfoList
    //PAM|ys|C
    hDevInfoSet=SetupDiGetClassDevs(&HidGuid,
                                    NULL,
                                    NULL,
                                    DIGCF_DEVICEINTERFACE|DIGCF_PRESENT);

    //AddToInfOut("}ld]");
    //M]ƶXCӳ]ƶiC|AˬdO_Oڭ̭n䪺]
    //ڭ̫w]ơAΪ̳]Ƥwgd䧹ɡANhXdC
    //VĤ@ӳ]ơAYNMemberIndexm0C
    MemberIndex=0;
    while(1) {
        //եSetupDiEnumDeviceInterfacesb]ƫHXs
        //MemberIndex]ƫHC
        Result=SetupDiEnumDeviceInterfaces(hDevInfoSet,
                                           NULL,
                                           &HidGuid,
                                           MemberIndex,
                                           &DevInterfaceData);

        //pGHѡAh]Ƥwgd䧹AhX`C
        if(Result==FALSE) break;

        //NMemberIndexVU@ӳ]
        MemberIndex++;

        //pGH\Ah~ӳ]ƪԲӫHCb]
        //ԲӫHɡAݭnDOsԲӫHݭnhjwİϡAoqL
        //Ĥ@եΨSetupDiGetDeviceInterfaceDetailCo
        //ѽwİϩM׳NULLѼơAôѤ@ӥΨӫOsݭnhj
        //wİϪܶqRequiredSizeC
        Result=SetupDiGetDeviceInterfaceDetail(hDevInfoSet,
                                               &DevInterfaceData,
                                               NULL,
                                               NULL,
                                               &RequiredSize,
                                               NULL);

        //MAt@ӤjpRequiredSizewİϡAΨӫOs]ƸԲӫHC
        pDevDetailData=(PSP_DEVICE_INTERFACE_DETAIL_DATA)malloc(RequiredSize);
        if(pDevDetailData==NULL) { //pGsAh^C
            //MessageBox("s!");
            SetupDiDestroyDeviceInfoList(hDevInfoSet);
            return FALSE;
        }

        //ó]mpDevDetailDatacbSizec骺jp]`NuOcjpA
        //]A᭱wİϡ^C
        pDevDetailData->cbSize=sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA);

        //MAեSetupDiGetDeviceInterfaceDetailƨ]ƪ
        //ԲӫHCoեγ]mϥΪwİϥHνwİϤjpC
        Result=SetupDiGetDeviceInterfaceDetail(hDevInfoSet,
                                               &DevInterfaceData,
                                               pDevDetailData,
                                               RequiredSize,
                                               NULL,
                                               NULL);

        //N]Ƹ|ƻsXӡAMPӽЪsC
        //MyDevPathName=pDevDetailData->DevicePath;
        _tcscpy_s(MyDevPathName, pDevDetailData->DevicePath);
        free(pDevDetailData);

        //pGեΥѡAhdU@ӳ]ơC
        if(Result==FALSE) continue;

        //pGեΦ\AhϥΤaŪgXݪCreateFile
        //]ƪݩʡA]AVIDBPIDBC
        //@ǿW]ơ]ҦpUSBL^AϥŪXݤ覡OLk}A
        //ӨϥΤaŪgXݪ榡~iH}oǳ]ơAq]ƪݩʡC
        hDevHandle=CreateFile(MyDevPathName,
                              NULL,
                              FILE_SHARE_READ|FILE_SHARE_WRITE,
                              NULL,
                              OPEN_EXISTING,
                              FILE_ATTRIBUTE_NORMAL,
                              NULL);

        //pG}\Ah]ݩʡC
        if(hDevHandle!=INVALID_HANDLE_VALUE) {
            //]ƪݩʨëOsbDevAttributesc餤
            Result=HidD_GetAttributes(hDevHandle,
                                      &DevAttributes);

            //襴}]
            CloseHandle(hDevHandle);

            //ѡAdU@
            if(Result==FALSE) continue;

            //pG\AhNݩʤVIDBPIDHγ]ƪPڭ̻ݭn
            //iApG@PܡAhNOڭ̭n䪺]ơC
            if(DevAttributes.VendorID == usVID
                    && ((DevAttributes.ProductID == usPID0)
                        || (DevAttributes.ProductID == usPID1)
                        || (DevAttributes.ProductID == usPID2))) {
                MyDevFound=TRUE; //]m]Ƥwg
                //AddToInfOut("]Ƥwg");

                if(bUseTwoHandle) {
                    //ôҪҵ豸ֱʹöдʽ֮
                    //ѡΪ첽ʷʽ
                    //ʽ豸
                    hReadHandle=CreateFile(MyDevPathName,
                                           GENERIC_READ,
                                           FILE_SHARE_READ|FILE_SHARE_WRITE,
                                           NULL,
                                           OPEN_EXISTING,
                                           //FILE_ATTRIBUTE_NORMAL|FILE_FLAG_OVERLAPPED,
                                           FILE_ATTRIBUTE_NORMAL,
                                           NULL);
                    //if(hReadHandle!=INVALID_HANDLE_VALUE)AddToInfOut("ʴ豸ɹ");
                    //else AddToInfOut("ʴ豸ʧ");

                    //дʽ豸
                    hWriteHandle=CreateFile(MyDevPathName,
                                            GENERIC_WRITE,
                                            FILE_SHARE_READ|FILE_SHARE_WRITE,
                                            NULL,
                                            OPEN_EXISTING,
                                            //FILE_ATTRIBUTE_NORMAL|FILE_FLAG_OVERLAPPED,
                                            FILE_ATTRIBUTE_NORMAL,
                                            NULL);
                    //if(hWriteHandle!=INVALID_HANDLE_VALUE)AddToInfOut("дʴ豸ɹ");
                    //else AddToInfOut("дʴ豸ʧ");
                } else {
                    hWriteHandle =
                        hReadHandle = CreateFile(MyDevPathName,
                                                 GENERIC_READ | GENERIC_WRITE,
                                                 FILE_SHARE_READ|FILE_SHARE_WRITE,
                                                 NULL,
                                                 OPEN_EXISTING,
                                                 //FILE_ATTRIBUTE_NORMAL|FILE_FLAG_OVERLAPPED,
                                                 FILE_ATTRIBUTE_NORMAL,
                                                 NULL);
                }

                m_hReadHandle.push_back(hReadHandle);
                m_hWriteHandle.push_back(hWriteHandle);
                m_sNames.push_back(MyDevPathName);
                //ֶ¼öָ̻߳СΪ֮ǰûе
                //ݵĺҲͲ¼ĲҪֶһ
                //¼öָ̻߳С
                //	SetEvent(ReadOverlapped.hEvent);

                //ʾ豸״̬
                //SetDlgItemText(IDC_DS,"豸Ѵ");

                //ҵ豸˳ѭֻһĿ豸ҵ˳
                //ˡҪеĿ豸гĻһ
                //飬ҵͱУֱ豸ϲ˳
                //break;
                //
            }
        }
        //pG}ѡAhdU@ӳ]
        else continue;
    }

    //եSetupDiDestroyDeviceInfoListƾP]ƫHX
    SetupDiDestroyDeviceInfoList(hDevInfoSet);

    this->SetActiveDevice(0);
    //pG]ƤwgAӨϯUާ@sAæPɸT}]ƫs
    return MyDevFound;
}


BOOL CHidIO::ReadFile(char *pcBuffer, size_t szMaxLen, DWORD *pdwLength, DWORD dwMilliseconds)
{
    if(m_szActiveDeviceIndex >= m_hReadHandle.size())
        return FALSE;
    HANDLE hReadHandle = m_hReadHandle[m_szActiveDeviceIndex];
    HANDLE events[2] = {m_hAbordEvent, m_hReadEvent};

    OVERLAPPED overlapped;
    memset(&overlapped, 0, sizeof(overlapped));
    overlapped.hEvent = m_hReadEvent;

    if(pdwLength != NULL)
        *pdwLength = 0;

    if(!::ReadFile(hReadHandle, pcBuffer, szMaxLen, NULL, &overlapped)) {
        DWORD dwError = ::GetLastError();
        if(dwError != ERROR_IO_PENDING) {
            return FALSE;
        }

        /* Wait for pending IO event */
        DWORD dwIndex = WaitForMultipleObjects(2, events, FALSE, dwMilliseconds);
        if(dwIndex == WAIT_OBJECT_0
                || dwIndex == WAIT_OBJECT_0 + 1) {
            ResetEvent(events[dwIndex - WAIT_OBJECT_0]);
            if(dwIndex == WAIT_OBJECT_0) {
                return FALSE;	//Abort event
            }
        } else {
            return FALSE;
        }
    }


    DWORD dwLength = 0;
    //Read OK
    GetOverlappedResult(hReadHandle, &overlapped, &dwLength, TRUE);
    if(pdwLength != NULL)
        *pdwLength = dwLength;

    return TRUE;
}

BOOL CHidIO::WriteFile(const char *pcBuffer, size_t szLen, DWORD *pdwLength, DWORD dwMilliseconds)
{
    if(m_szActiveDeviceIndex >= m_hWriteHandle.size())
        return FALSE;
    HANDLE hWriteHandle = m_hWriteHandle[m_szActiveDeviceIndex];
    HANDLE events[2] = {m_hAbordEvent, m_hWriteEvent};

    OVERLAPPED overlapped;
    memset(&overlapped, 0, sizeof(overlapped));
    overlapped.hEvent = m_hWriteEvent;

    if(pdwLength != NULL)
        *pdwLength = 0;

    if(!::WriteFile(hWriteHandle, pcBuffer, szLen, NULL, &overlapped)) {
        if(::GetLastError() != ERROR_IO_PENDING)
            return FALSE;

        DWORD dwIndex = WaitForMultipleObjects(2, events, FALSE, dwMilliseconds);
        if(dwIndex == WAIT_OBJECT_0
                || dwIndex == WAIT_OBJECT_0 + 1) {
            ResetEvent(events[dwIndex - WAIT_OBJECT_0]);

            if(dwIndex == WAIT_OBJECT_0)
                return FALSE;	//Abort event
        } else
            return FALSE;
    }

    DWORD dwLength = 0;
    //Write OK
    GetOverlappedResult(hWriteHandle, &overlapped, &dwLength, TRUE);
    if(pdwLength != NULL)
        *pdwLength = dwLength;

    return TRUE;
}
